/*
 * Copyright SpringSource Inc 2007,2009.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.springsource.util.math;

import java.util.Set;

/**
 * Provide a mathematical relation which is notionally a set of ordered pairs. It represents a relationship between the
 * members of two sets: the domain and the range.
 * 
 * @author Glyn Normington
 * 
 * @param <X> the type of elements in the domain of the relation
 * @param <Y> the type of elements in the range of the relation
 */
public interface ConcurrentRelation<X, Y> {

    /**
     * Add the given pair (x, y) to the relation. It is not an error if the pair is already present.
     * 
     * @param x
     * @param y
     * @return true if and only if the relation was updated
     */
    boolean add(X x, Y y);

    /**
     * Remove the given pair (x, y) from the relation. It is not an error if the pair is not present.
     * 
     * @param x
     * @param y
     * @return true if and only if the relation was updated
     */
    boolean remove(X x, Y y);

    /**
     * Return true if and only if the given pair (x, y) is present in the relation.
     * 
     * @param x
     * @param y
     * @return true if the pair is present in the relation
     */
    boolean contains(X x, Y y);

    /**
     * Return the domain of the relation.
     * 
     * @return a set comprising the domain
     */
    Set<X> dom();

    /**
     * Return the range of the relation.
     * 
     * @return a set comprising the range
     */
    Set<Y> ran();

    /**
     * Return the relational image of a set of X's.
     * 
     * @param xset a set of X's
     * @return the set of Y's which relate to X's in xset
     */
    Set<Y> relationalImage(Set<X> xset);

    /**
     * Remove all the pairs (x, y) where x is in the given set of X's.
     * 
     * @param xset a set of X's
     */
    void domSubtract(Set<X> xset);
}
