/*
 * Copyright SpringSource Inc 2007,2009.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.springsource.util.math;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * Defines common operations on {@link Set Sets}.<p/>
 * 
 * <strong>Concurrent Semantics</strong><br />
 * 
 * Threadsafe.
 * 
 * @author Rob Harrop
 * @since 1.0
 */
public class Sets {

    /**
     * Calculates the difference between <code>a</code> and <code>b</code>.
     * 
     * @param <T> the type of the <code>Set</code>.
     * @param a the lhs of the difference.
     * @param b the rhs of the difference.
     * @return the difference between <code>a</code> and <code>b</code>.
     */
	public static <T> Set<T> difference(Set<T> a, Set<T> b) {
        Set<T> result = new HashSet<T>(a);
        result.removeAll(b);
        return result;
    }

    /**
     * Calculates the intersection between <code>a</code> and <code>b</code>.
     * 
     * @param <T> the type of the <code>Set</code>.
     * @param a the lhs of the intersection.
     * @param b the rhs of the intersection.
     * @return the intersection between <code>a</code> and <code>b</code>.
     */
    public static <T> Set<T> intersection(Set<T> a, Set<T> b) {
        Set<T> iter = a.size() < b.size() ? a : b;
        Set<T> other = iter == a ? b : a;

        Set<T> result = new HashSet<T>();
        for (T t : iter) {
            if (other.contains(t)) {
                result.add(t);
            }
        }
        return result;
    }

    /**
     * Creates a <code>Set</code> containing the supplied items.
     * 
     * @param <T> the type of the <code>Set</code> entries.
     * @param items the <code>Set</code> contents.
     * @return the resultant <code>Set</code>.
     */
    public static <T> Set<T> asSet(T... items) {
        Set<T> set = new HashSet<T>(items.length);
        Collections.addAll(set, items);
        return set;
    }
}
